function [p, resNorm, RESIDUAL, imageOfFit, weighted, paramNames] = myGauss2DFit_rotation2_mixture(I,indices,initialGuess,lb, ub, W, maxPositionDisplacement, forcePeakAtMaxIntensity)

%I            - the image
%indices      - [ymin, ymax, xmin, xmax], the mesh will be generated by ymin:ymax and xmin:xmax
%initialGuess - [meanx meany sigmax sigmay cMult cAdd rho]
%the target function to be fitted is f(x,y) = cMult1 * Gauss(x,y;mx1,my1,sx1,sy1,alpha1) + cMult2 * Gauss(x,y;mx2,my2,sx2,sy2,alpha2) + cAdd;
%
%output   %         1       2        3          4        5        6        7        8        9        10        11        12      13
% p = [meanx(1) meany(1) sigmax(1) sigmay(1) cMult(1) alpha(1) meanx(2) meany(2) sigmax(2) sigmay(2) cMult(2) alpha(2) cAdd]
% resNorm(1) ==> resNorm, resNorm(2) ==> gSNR

%see also: myGauss2D_rotation2_mixture, myGauss2DFit_rotation, myGaussianMixture2DFit, myGauss2D_rotation2
%                1        2        3         4        5        6        7        8         9        10        11       12      13
paramNames = {'meanx1' 'meany1' 'sigmax1' 'sigmay1' 'cMult1' 'alpha1' 'meanx2' 'meany2' 'sigmax2' 'sigmay2' 'cMult2' 'alpha2' 'cAdd'};
forcePeakAtMaxIntensity = 0; %input variable necessary for function signature!

objectiveFunction = @myGauss2D_rotation2_mixture;
global param;

    function D = diff_objectiveFunction_image(p, grid, I)
        D = objectiveFunction(p, grid) - I;
    end
    function D = diff_objectiveFunction_image_weighted(p, grid, I)
        D = diff_objectiveFunction_image(p, grid, I) .* W;
    end

if(nargin < 2 || isempty(indices))
    [X,Y] = meshgrid(1:size(I,2),1:size(I,1));
else
    [X,Y] = meshgrid(indices(3):indices(4),indices(1):indices(2));
end

grid = [X Y];
xmin = X(1);
ymin = Y(1);
xmax = X(end);
ymax = Y(end);

if(nargin >= 6 && ~isempty(W))
    functionToBeFitted = @diff_objectiveFunction_image_weighted;
    weighted = 1;
else
    functionToBeFitted = @diff_objectiveFunction_image;
    weighted = 0;
end

if(nargin < 7 || isempty(maxPositionDisplacement))
    try
        maxPositionDisplacement = param.ap.myGauss2DFit.maxPositionDisplacement;
    catch
        maxPositionDisplacement = [inf inf];
        printDebugStack(lasterror);
    end
end
if(length(maxPositionDisplacement) == 1)
    maxPositionDisplacement(2) = maxPositionDisplacement(1);
elseif(isempty(maxPositionDisplacement))
    maxPositionDisplacement = [inf inf];
end
if(length(maxPositionDisplacement) ~= 2)
    error('maxPositionDisplacement must have too elements at this point in the code');
end

minInt = min(I(:));
maxInt = max(I(:));

if(nargin < 3 || isempty(initialGuess))
    localMaxima = imregionalmax(I);
    localMaximaIdx = find(localMaxima(:));
    if(length(localMaximaIdx) < 2)
        error('no two local maxima found in image. cannot fit a two-Gaussian mixture');
    end
    
    [localMaxIntensities,ix ] = sort(I(localMaximaIdx),'descend');
    localMaximaIdx = localMaximaIdx(ix);
    
    [my1,mx1] = ind2sub(size(I), localMaximaIdx(1));
    [my2,mx2] = ind2sub(size(I), localMaximaIdx(2));
    
    mx1_inGridCoord = mx1+xmin-1;
    my1_inGridCoord = my1+ymin-1;
    mx2_inGridCoord = mx2+xmin-1;
    my2_inGridCoord = my2+ymin-1;
    
    %                      mx(1)             my(1)      sx(1)  sy(1)          cMult(1)    alpha(1)          mx(2)             my(2)        sx(2)  sy(2)        cMult(2)   alpha(2)  cAdd
    initialGuess = [mx1_inGridCoord,   my1_inGridCoord,   2,     2,     I(my1,mx1)-minInt,  0,         mx2_inGridCoord,  my2_inGridCoord,   2,     2,    I(my2,mx2)-minInt, 0,     minInt];
    if(getDebugLevel()>=2)
        figure, imshow(I,[]), hold on, plot(mx1,my1, 'xr'),plot(mx2,my2, 'dg')
        title('position of the two highest local maxima')
    end
end

try
    minSigma                = param.ap.myGauss2DFit.minSigma / param.resolution;
    maxSigma                = param.ap.myGauss2DFit.maxSigma / param.resolution;
    if(nargin < 4 || strcmpi(lb,'auto'))
        %     mx(1)   my(1)    sx(1)     sy(1)    cMult(1) alpha(1)    mx(2)  my(2)   sx(2)      sy(2)   cMult(2)  alpha(2)   cAdd
        lb = [xmin,  ymin,   minSigma,  minSigma,  minInt,    -90   , xmin,  ymin,  minSigma, minSigma,     0,     -90,    minInt];
    end
    
    if(nargin < 5 || strcmpi(ub,'auto'))
        %     mx(1)   my(1)    sx(1)     sy(1)     cMult(1) alpha(1)  mx(2)  my(2)   sx(2)     sy(2)   cMult(2)  alpha(2)   cAdd
        ub = [xmax,   ymax,   maxSigma,  maxSigma,  3*maxInt,   90  , xmax,  ymax,  maxSigma, maxSigma, 3*maxInt,    90,   maxInt];
    end
catch ME
    printDebugStack(ME);
    lb = [];
    ub = [];
end


curvefitoptions = optimset('Display','off');
[p,resNorm,RESIDUAL] = lsqnonlin(functionToBeFitted,initialGuess,lb,ub,curvefitoptions,grid,I);

% compute SNR
try
    imageOfFit{1} = objectiveFunction( p, grid );
    bg = p(13);
    peak1 = p(5);
    peak2 = p(11);
    I_minusBG = I - bg;
    intensityThr = min([peak1 peak2] / 10) + bg;
    maskSignal = imageOfFit{1} >= intensityThr;
    maskBackground = ~maskSignal;
    [SNR, r, signal, noise] = ComputeSNR(I_minusBG, maskSignal, maskBackground);
catch
    r = -1;
end
resNorm(2) = r;

if(nargout > 3)
    imageOfFit{2} = objectiveFunction( [p(1:6) 0 0 0 0 0 0 p(13)], grid );
    imageOfFit{3} = objectiveFunction( [0 0 0 0 0 0 p(7:13)], grid );
end

end

